.macro ClearVRAM
  lda #$80
  sta $2115		;Set VRAM port to word access
  ldx #$1809
  stx $4300		;Set DMA mode to fixed source, WORD to $2118/9
  ldx #$0000
  stx $2116		;Set VRAM port address to $0000
  ldx #.LOWORD(zero_fill_byte)
  stx $4302		;Set source address to $xx:0000
  lda #.BANKBYTE(zero_fill_byte)
  sta $4304		;Set source bank
  ldx #$0000
  stx $4305		;Set transfer size to 65536 bytes
  lda #$01
  sta $420B		;Initiate transfer
.endmacro

;=========================================================================================

.macro ClearPalette
  stz $2121
  ldx #$0100

clear_palette_loop:
  stz $2122
  stz $2122
  dex
  bne clear_palette_loop
.endmacro

;=========================================================================================

.macro InitializeSNES
  sei		;Disable interrupts
  clc		;Clear carry, used by XCE
  xce		;Set 65816 native mode
  jmp :+	;Needed to set K (bank of PC) properly if MODE 21 is ever used;
		;see official SNES developers docs, "Programming Cautions"

: cld		;Disable decimal mode
  phk		;Push K (PC bank)
  plb		;Pull B (data bank, i.e. data bank now equals PC bank)

  rep #$30	;A=16, X/Y=16

;Note: this should correlate with ZEROPAGE in lorom256k.cfg
  lda #$0000
  tcd		;Set D = $0000 (direct page)

;Note: this should correlate with the top of BSS in lorom256k.cfg
  ldx #$1fff
  txs		;Set X = $1fff (stack pointer)

;Register initialisation values, per official Nintendo documentation
;Note: if you are looking at this code and thinking "how can I optimise"
;this?", then you are doing nothing useful with your time.  This code is
;called ONCE on reset or power-on, it does not need to be optimised.  I
;(koitsu) find it remarkable how many people have tried to "optimise"
;this routine.  STOP IT.

  sep #$20	;A=8
  lda #$80
  sta $2100
  stz $2101
  stz $2102
  stz $2103
  stz $2104
  stz $2105
  stz $2106
  stz $2107
  stz $2108
  stz $2109
  stz $210a
  stz $210b
  stz $210c
  stz $210d
  stz $210d
  stz $210e
  stz $210e
  stz $210f
  stz $210f
  stz $2110
  stz $2110
  stz $2111
  stz $2111
  stz $2112
  stz $2112
  stz $2113               
  stz $2113               
  stz $2114
  stz $2114
  lda #$80
  sta $2115
  stz $2116
  stz $2117
  stz $211a
  stz $211b
  lda #$01
  sta $211b
  stz $211c
  stz $211c
  stz $211d
  stz $211d
  stz $211e       
  lda #$01
  sta $211e
  stz $211f
  stz $211f
  stz $2120
  stz $2120
  stz $2121
  stz $2123
  stz $2124
  stz $2125
  stz $2126
  stz $2127
  stz $2128
  stz $2129
  stz $212a
  stz $212b
  stz $212c
  stz $212d
  stz $212e
  stz $212f
  stz $4200
  lda #$ff
  sta $4201
  stz $4202
  stz $4203
  stz $4204
  stz $4205
  stz $4206
  stz $4207
  stz $4208
  stz $4209
  stz $420a
  stz $420b
  stz $420c
  stz $420d

  ClearVRAM		;Zero VRAM. Requires A=8, X/Y=16
  ClearPalette		;Zero palette. Requires A=8, X/Y=16
  
;clear sprite tables

  stz $2102	;sprites initialized to be off the screen, palette 0, character 0
  stz $2103
  ldx #$0080
  lda #$F0

loop08:
  sta $2104	;set X = 240
  sta $2104	;set Y = 240
  sta $2104	;set character = $00
  sta $2104	;set priority=0, no flips
  dex
  bne loop08
  ldx #$0020

loop09:
  stz $2104		;set size bit=0, x MSB = 0
  dex
  bne loop09

;clear SNES RAM

  stz $2181	;set WRAM address to $000000
  stz $2182
  stz $2183

  ldx #$8008
  stx $4300		;Set DMA mode to fixed source, BYTE to $2180
  ldx #.LOWORD(zero_fill_byte)
  stx $4302		;Set source offset
  lda #.BANKBYTE(zero_fill_byte)
  sta $4304		;Set source bank
  ldx #$0000
  stx $4305		;Set transfer size to 64KBytes (65536 bytes)
  lda #$01
  sta $420B		;Initiate transfer

  lda #$01		;now zero the next 64KB (i.e. 128KB total)
  sta $420B		;Initiate transfer
.endmacro

;Byte used to zero/initialise all of memory in above macro
;This is needed because DMA requires an address of a byte to
;read from, not a raw value.

.segment "RODATA"
zero_fill_byte:
  .byte $00